@@ -1,5 +1,5 @@
 print "Using native SQLite3\n"
-require 'fixtures/course'
+require_dependency 'fixtures/course'
 require 'logger'
 ActiveRecord::Base.logger = Logger.new("debug.log")
 
@@ -18,7 +18,7 @@ def make_connection(clazz, db_file, db_definitions_file)
     raise SqliteError.new("Seems that there is no sqlite3 executable available") unless system(sqlite_command)
     clazz.establish_connection(
         :adapter => "sqlite3",
-        :dbfile  => db_file)
+        :database  => db_file)
     script = File.read("#{BASE_DIR}/db_definitions/#{db_definitions_file}")
     # SQLite-Ruby has problems with semi-colon separated commands, so split and execute one at a time
     script.split(';').each do
@@ -28,9 +28,10 @@ def make_connection(clazz, db_file, db_definitions_file)
   else
     clazz.establish_connection(
         :adapter => "sqlite3",
-        :dbfile  => db_file)
+        :database  => db_file)
   end
 end
 
 make_connection(ActiveRecord::Base, sqlite_test_db, 'sqlite.sql')
 make_connection(Course, sqlite_test_db2, 'sqlite2.sql')
+load(File.join(BASE_DIR, 'db_definitions', 'schema.rb'))
