@@ -55,6 +55,34 @@ module Rails
         def readme(*args)
         end
 
+        protected
+          def migration_directory(relative_path)
+            directory(@migration_directory = relative_path)
+          end
+
+          def existing_migrations(file_name)
+            Dir.glob("#{@migration_directory}/[0-9]*_*.rb").grep(/[0-9]+_#{file_name}.rb$/)
+          end
+
+          def migration_exists?(file_name)
+            not existing_migrations(file_name).empty?
+          end
+
+          def current_migration_number
+            Dir.glob("#{@migration_directory}/[0-9]*.rb").inject(0) do |max, file_path|
+              n = File.basename(file_path).split('_', 2).first.to_i
+              if n > max then n else max end
+            end
+          end
+
+          def next_migration_number
+            current_migration_number + 1
+          end
+
+          def next_migration_string(padding = 3)
+            "%.#{padding}d" % next_migration_number
+          end
+
         private
           # Ask the user interactively whether to force collision.
           def force_file_collision?(destination)
@@ -84,10 +112,10 @@ module Rails
             begin_mark = template_part_mark(template_options[:begin_mark], template_options[:mark_id])
             end_mark = template_part_mark(template_options[:end_mark], template_options[:mark_id])
             begin_mark + rendered_part + end_mark
-         end
+          end
 
-         def template_part_mark(name, id)
-           "<!--[#{name}:#{id}]-->\n"
+          def template_part_mark(name, id)
+            "<!--[#{name}:#{id}]-->\n"
           end
       end
 
@@ -135,26 +163,34 @@ module Rails
 
         # Copy a file from source to destination with collision checking.
         #
-        # The file_options hash accepts :chmod and :shebang options.
+        # The file_options hash accepts :chmod and :shebang and :collision options.
         # :chmod sets the permissions of the destination file:
         #   file 'config/empty.log', 'log/test.log', :chmod => 0664
         # :shebang sets the #!/usr/bin/ruby line for scripts
         #   file 'bin/generate.rb', 'script/generate', :chmod => 0755, :shebang => '/usr/bin/env ruby'
+        # :collision sets the collision option only for the destination file: 
+        #   file 'settings/server.yml', 'config/server.yml', :collision => :skip
         #
         # Collisions are handled by checking whether the destination file
         # exists and either skipping the file, forcing overwrite, or asking
         # the user what to do.
-        def file(relative_source, relative_destination, file_options = {})
+        def file(relative_source, relative_destination, file_options = {}, &block)
           # Determine full paths for source and destination files.
-          source      = source_path(relative_source)
-          destination = destination_path(relative_destination)
+          source              = source_path(relative_source)
+          destination         = destination_path(relative_destination)
+          destination_exists  = File.exists?(destination)
+
+          # If source and destination are identical then we're done.
+          if destination_exists and identical?(source, destination, &block)
+            return logger.identical(relative_destination) 
+          end
 
           # Check for and resolve file collisions.
-          if File.exists?(destination)
+          if destination_exists
 
             # Make a choice whether to overwrite the file.  :force and
             # :skip already have their mind made up, but give :ask a shot.
-            choice = case options[:collision].to_sym #|| :ask
+            choice = case (file_options[:collision] || options[:collision]).to_sym #|| :ask
               when :ask   then force_file_collision?(relative_destination)
               when :force then :force
               when :skip  then :skip
@@ -181,17 +217,16 @@ module Rails
           # if block given so templaters may render the source file.  If a
           # shebang is requested, replace the existing shebang or insert a
           # new one.
-          File.open(destination, 'w') do |df|
-            File.open(source) do |sf|
+          File.open(destination, 'wb') do |df|
+            File.open(source, 'rb') do |sf|
               if block_given?
                 df.write(yield(sf))
               else
-                line = sf.gets
                 if file_options[:shebang]
                   df.puts("#!#{file_options[:shebang]}")
-                  df.puts(line) if line !~ /^#!/
-                else
-                  df.puts(line)
+                  if line = sf.gets
+                    df.puts(line) if line !~ /^#!/
+                  end
                 end
                 df.write(sf.read)
               end
@@ -202,6 +237,19 @@ module Rails
           if file_options[:chmod]
             FileUtils.chmod(file_options[:chmod], destination)
           end
+          
+          # Optionally add file to subversion
+          system("svn add #{destination}") if options[:svn]
+        end
+
+        # Checks if the source and the destination file are identical. If
+        # passed a block then the source file is a template that needs to first
+        # be evaluated before being compared to the destination.
+        def identical?(source, destination, &block)
+          return false if File.directory? destination
+          source      = block_given? ? File.open(source) {|sf| yield(sf)} : IO.read(source)
+          destination = IO.read(destination)
+          source == destination
         end
 
         # Generate a file for a Rails application using an ERuby template.
@@ -246,6 +294,9 @@ module Rails
           else
             logger.create relative_path
             FileUtils.mkdir_p(path) unless options[:pretend]
+            
+            # Optionally add file to subversion
+            system("svn add #{path}") if options[:svn]
           end
         end
 
@@ -257,6 +308,14 @@ module Rails
           end
         end
 
+        # When creating a migration, it knows to find the first available file in db/migrate and use the migration.rb template.
+        def migration_template(relative_source, relative_destination, template_options = {})
+          migration_directory relative_destination
+          migration_file_name = template_options[:migration_file_name] || file_name
+          raise "Another migration is already named #{migration_file_name}: #{existing_migrations(migration_file_name).first}" if migration_exists?(migration_file_name)
+          template(relative_source, "#{relative_destination}/#{next_migration_string}_#{migration_file_name}.rb", template_options)
+        end
+
         private
           # Raise a usage error with an informative WordNet suggestion.
           # Thanks to Florian Gross (flgr).
@@ -294,11 +353,26 @@ end_message
       # manifest and attempt to completely erase the results of each action.
       class Destroy < RewindBase
         # Remove a file if it exists and is a file.
-        def file(relative_source, relative_destination, options = {})
+        def file(relative_source, relative_destination, file_options = {})
           destination = destination_path(relative_destination)
           if File.exists?(destination)
             logger.rm relative_destination
-            FileUtils.rm(destination) unless options[:pretend]
+            unless options[:pretend]
+              if options[:svn]
+                # If the file has been marked to be added
+                # but has not yet been checked in, revert and delete
+                if options[:svn][relative_destination]
+                  system("svn revert #{destination}")
+                  FileUtils.rm(destination)
+                else
+                # If the directory is not in the status list, it
+                # has no modifications so we can simply remove it
+                  system("svn rm #{destination}")
+                end  
+              else
+                FileUtils.rm(destination)
+              end
+            end
           else
             logger.missing relative_destination
             return
@@ -319,7 +393,22 @@ end_message
             if File.exists?(path)
               if Dir[File.join(path, '*')].empty?
                 logger.rmdir partial
-                FileUtils.rmdir(path) unless options[:pretend]
+                unless options[:pretend]
+                  if options[:svn]
+                    # If the directory has been marked to be added
+                    # but has not yet been checked in, revert and delete
+                    if options[:svn][relative_path]
+                      system("svn revert #{path}")
+                      FileUtils.rmdir(path)
+                    else
+                    # If the directory is not in the status list, it
+                    # has no modifications so we can simply remove it
+                      system("svn rm #{path}")
+                    end
+                  else
+                    FileUtils.rmdir(path)
+                  end
+                end
               else
                 logger.notempty partial
               end
@@ -333,6 +422,22 @@ end_message
         def complex_template(*args)
           # nothing should be done here
         end
+
+        # When deleting a migration, it knows to delete every file named "[0-9]*_#{file_name}".
+        def migration_template(relative_source, relative_destination, template_options = {})
+          migration_directory relative_destination
+
+          migration_file_name = template_options[:migration_file_name] || file_name
+          unless migration_exists?(migration_file_name)
+            puts "There is no migration named #{migration_file_name}"
+            return
+          end
+
+
+          existing_migrations(migration_file_name).each do |file_path|
+            file(relative_source, file_path, template_options)
+          end
+        end
       end
 
 
@@ -365,6 +470,11 @@ end_message
         def readme(*args)
           logger.readme args.join(', ')
         end
+        
+        def migration_template(relative_source, relative_destination, options = {})
+          migration_directory relative_destination
+          logger.migration_template file_name
+        end
       end
 
       # Update generator's action manifest.
